from fudge.inspector import arg
from ..._utils import *

from .youtube import *
from .vimeo import *

from ....video import backends


class TestableBackend(object):
    pass


class SomeOtherTestableBackend(object):
    pass


class GetBackendTestCase(TestCase):
    def backend_name(self, name):
        return "%s.%s" % (self.__module__, name)

    def test_uses_backend_from_settings(self):
        backend_name = self.backend_name('TestableBackend')
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=backend_name)

        with fudge.patched_context(backends, 'default_settings', settings):
            backend = backends.get_backend()
            self.assertIsA(backend, TestableBackend)

    def test_uses_injected_settings_if_provided(self):
        backend_name = self.backend_name('SomeOtherTestableBackend')
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=backend_name)

        fudge.clear_calls()

        backend = backends.get_backend(settings=settings)
        self.assertIsA(backend, SomeOtherTestableBackend)

    def test_returns_MultipleBackend_if_configured_with_a_list(self):
        my_backends = [self.backend_name('TestableBackend'),
                self.backend_name('SomeOtherTestableBackend')]

        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=my_backends)

        backend = backends.get_backend(settings=settings)
        self.assertIsA(backend, backends.MultipleBackends)

    def test_MultipleBackend_is_passed_configured_objects(self):

        def is_TestableBackend(obj):
            return isinstance(obj, TestableBackend)

        def is_SomeOtherTestableBackend(obj):
            return isinstance(obj, SomeOtherTestableBackend)

        fake = fudge.Fake(backends.MultipleBackends)
        fake.expects('__init__').with_args(arg.passes_test(is_TestableBackend),
                arg.passes_test(is_SomeOtherTestableBackend))

        my_backends = [self.backend_name('TestableBackend'),
                self.backend_name('SomeOtherTestableBackend')]
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EXTERNAL_VIDEO_BACKEND=my_backends)

        with fudge.patched_context(backends, 'MultipleBackends', fake):
            backends.get_backend(settings=settings)


class NeverMatchesBackend(object):
    def prepare(self, video):
        return None


class AlwayMatchesBackend(object):
    def prepare(self, video):
        video.url, video.id = video.raw_url.split("?")
        return True


class MultipleBackendsTestCase(TestCase):
    def test_each_backend_is_called_until_one_answers_with_non_none(self):
        backend = backends.MultipleBackends(NeverMatchesBackend(),
                AlwayMatchesBackend())
        video = EmbeddedVideo(backend=backend)

        first = "first-%d" % random.randint(100, 200)
        second = "second-%d" % random.randint(100, 200)
        video.raw_url = "%s?%s" % (first, second)

        self.assertTrue(backend.prepare(video))
        self.assertEqual(first, video.url)
        self.assertEqual(second, video.id)

    def test_returns_not_True_if_no_backends_match(self):
        backend = backends.MultipleBackends(NeverMatchesBackend())
        video = EmbeddedVideo(backend=backend)
        video.raw_url = "some-random-url"

        self.assertFalse(backend.prepare(video))

    def test_sets_backend_on_EmbeddedVideo_to_the_backend_that_was_found(self):
        backend = backends.MultipleBackends(NeverMatchesBackend(),
                AlwayMatchesBackend())
        video = EmbeddedVideo(backend=backend)

        first = "first-%d" % random.randint(100, 200)
        second = "second-%d" % random.randint(100, 200)
        video.raw_url = "%s?%s" % (first, second)

        backend.prepare(video)
        self.assertIsA(video.backend, AlwayMatchesBackend)



import pkg_resources
pkg_resources.declare_namespace(__name__)

from django.conf import settings as default_settings
from django.utils.importlib import import_module

from .youtube import YouTubeBackend
from .vimeo import VimeoBackend


class MultipleBackends(object):
    def __init__(self, *others):
        self.others = others

    def prepare(self, video):
        for other in self.others:
            if other.prepare(video) is not None:
                video.backend = other
                return True


def get_backend(settings=None):
    if not settings:
        settings = default_settings

    backend_name = settings.ARMSTRONG_EXTERNAL_VIDEO_BACKEND

    def to_backend(a):
        module, backend_class = a.rsplit(".", 1)
        backend_module = import_module(module)
        return getattr(backend_module, backend_class)()

    if type(backend_name) is str:
        # TODO: Should raise an ImproperlyConfigured error if this isn't
        #       present in the settings variable.
        return to_backend(settings.ARMSTRONG_EXTERNAL_VIDEO_BACKEND)
    else:
        return MultipleBackends(*[to_backend(a) for a in
            settings.ARMSTRONG_EXTERNAL_VIDEO_BACKEND])



from datetime import datetime
from django.contrib.auth.models import User
from django.core.management.color import no_style
from django.db import connection
from django.db import models
from django.test import TestCase as DjangoTestCase
from django.utils import unittest
import fudge
import random

from .arm_content_support.models import Article, Video
from ..mixins.publication import PUB_STATUSES


def create_concrete_table(func=None, model=None):
    style = no_style()
    seen_models = connection.introspection.installed_models(
            connection.introspection.table_names())

    def actual_create(model):
        sql, _references = connection.creation.sql_create_model(model, style,
                seen_models)
        cursor = connection.cursor()
        for statement in sql:
            cursor.execute(statement)

    if func:
        def inner(self, *args, **kwargs):
            func(self, *args, **kwargs)
            actual_create(self.model)
        return inner
    elif model:
        actual_create(model)


def destroy_concrete_table(func=None, model=None):
    style = no_style()
    # Assume that there are no references to destroy, these are supposed to be
    # simple models
    references = {}

    def actual_destroy(model):
        sql = connection.creation.sql_destroy_model(model, references, style)
        cursor = connection.cursor()
        for statement in sql:
            cursor.execute(statement)

    if func:
        def inner(self, *args, **kwargs):
            func(self, *args, **kwargs)
            actual_destroy(self.model)
        return inner
    elif model:
        actual_destroy(model)


# TODO: pull into a common dev package so all armstrong code can use it
def concrete(klass):
    attrs = {'__module__': concrete.__module__, }
    while True:
        num = random.randint(1, 10000)
        if num not in concrete.already_used:
            break
    return type("Concrete%s%d" % (klass.__name__, num), (klass, ), attrs)
concrete.already_used = []


class TestCase(DjangoTestCase):
    def setUp(self):
        fudge.clear_expectations()
        fudge.clear_calls()

    def assertRelatedTo(self, model, field_name, related_model, many=False):
        if many is False:
            through = models.ForeignKey
        else:
            through = models.ManyToManyField

        # sanity check
        self.assertModelHasField(model, field_name, through)

        field = model._meta.get_field_by_name(field_name)[0]
        self.assertEqual(field.rel.to, related_model)

    def assertModelHasField(self, model, field_name, field_class=None):
        msg = "%s does not have a field named %s" % (model.__class__.__name__,
                field_name)
        self.assertTrue(hasattr(model, field_name), msg=msg)
        field = model._meta.get_field_by_name(field_name)[0]
        if field_class is not None:
            msg = "%s.%s is not a %s" % (model.__class__.__name__, field_name,
                    field_class.__class__.__name__)
            self.assertTrue(isinstance(field, field_class), msg=msg)

    def assertNone(self, obj, **kwargs):
        self.assertTrue(obj is None, **kwargs)

    def assertIsA(self, obj, cls, **kwargs):
        self.assertTrue(isinstance(obj, cls), **kwargs)

    def assertDoesNotHave(self, obj, attr, **kwargs):
        self.assertFalse(hasattr(obj, attr), **kwargs)


def create_random_article(**options):
    random_int = random.randint(1000, 9999)
    data = {
        'pub_date': datetime.now(),
        'pub_status': PUB_STATUSES['Published'],
        'title': 'Random Article %s' % random_int,
        'body': str(random_int),
    }
    data.update(options)
    return Article.objects.create(**data)


def create_random_video(**options):
    random_int = random.randint(1000, 9999)
    data = {
        'pub_date': datetime.now(),
        'pub_status': PUB_STATUSES['Published'],
        'title': 'Random Article %s' % random_int,
        'youtube_id': str(random_int),
    }
    data.update(options)
    return Video.objects.create(**data)


def generate_random_user():
    r = random.randint(10000, 20000)
    return User.objects.create(username="random-user-%d" % r,
            first_name="Some", last_name="Random User %d" % r)


def generate_random_staff_users(n=2):
    orig_users = generate_random_users(n)
    users = User.objects.filter(pk__in=[a.id for a in orig_users])
    users.update(is_staff=True)
    return [a for a in users]


class generate_random_staff_usersTestCase(TestCase):
    def test_returns_2_users_by_default(self):
        self.assertEqual(len(generate_random_staff_users()), 2)

    def test_returns_n_users(self):
        r = random.randint(1, 5)
        self.assertEqual(len(generate_random_staff_users(r)), r)

    def test_all_users_are_staff(self):
        users = generate_random_staff_users()
        for user in users:
            self.assertTrue(user.is_staff)


def generate_random_users(n=2):
    return [generate_random_user() for i in range(n)]


def add_n_users_to_authors(authors, *users):
    for user in users:
        authors.users.add(user)


def add_profile_to(profile_class, *users):
    for user in users:
        profile = profile_class.objects.create(user=user)
        user._profile_cache = profile


def add_authors_to(model, *authors):
    for author in authors:
        model.authors.add(author)


def random_authored_model(klass, *authors):
    article = klass.objects.create()
    add_authors_to(article, *authors)
    return article



from django.contrib.auth.models import User
from django.db import models
from polymorphic import PolymorphicModel

from ...fields import AuthorsField
from ...fields import EmbeddedVideoField
from ... import mixins
from ...mixins import PublicationMixin
from ...models import ContentBase


# for backwards compatibility -- should be removed
class BaseContent(PolymorphicModel, PublicationMixin):
    title = models.CharField(max_length=255)


class ConcreteContent(ContentBase):
    pass


class ConcreteArticle(ConcreteContent):
    pass


class ConcreteCommentary(ConcreteContent):
    pass


class Article(BaseContent):
    body = models.TextField()


class Video(BaseContent):
    youtube_id = models.CharField(max_length=30)


class SimpleVideoModel(models.Model):
    source = EmbeddedVideoField()


class SimpleMixedinVideoModel(mixins.EmbeddedVideoMixin, models.Model):
    pass


class SimpleAuthoredModel(models.Model):
    authors = AuthorsField()


class SimpleMixedinAuthorModel(mixins.AuthorsMixin, models.Model):
    pass


class AuthoredModelWithContentionalOverride(models.Model):
    authors = AuthorsField()
    authors_override = models.CharField(max_length=100)


class AuthoredModelWithConfiguredOverride(models.Model):
    authors = AuthorsField(override_field_name='custom_override')
    custom_override = models.CharField(max_length=100)


class AuthoredModelWithContentionalExtra(models.Model):
    authors = AuthorsField()
    authors_extra = models.CharField(max_length=100)


class AuthoredModelWithConfiguredExtra(models.Model):
    authors = AuthorsField(extra_field_name='custom_extra')
    custom_extra = models.CharField(max_length=100)


class SimpleProfile(models.Model):
    user = models.OneToOneField(User)

    def get_absolute_url(self):
        return '/%s/' % self.user.get_full_name().lower().replace(' ', '-')



from ._utils import TestCase, create_random_article, create_random_video
from .arm_content_support.models import BaseContent, Article, Video
from .admin import *
from .fields import *
from .mixins import *
from .models import *
from .video import *


class BaseContentTestCase(TestCase):
    def test_can_add_content(self):
        article = create_random_article()
        Article.objects.get(id=article.id)

    def test_can_fetch_combined_queryset(self):
        create_random_article()
        create_random_video()
        create_random_article()
        create_random_video()

        self.assertEqual(BaseContent.objects.count(), 4)
        for item in BaseContent.objects.all():
            self.assertFalse(item.__class__ == BaseContent)

        articles = [item for item in BaseContent.objects.all() if
            item.__class__ == Article]
        self.assertEqual(len(articles), 2)

        videos = [item for item in BaseContent.objects.all() if
            item.__class__ == Video]
        self.assertEqual(len(videos), 2)



from django.db import models
from django.db.models.query import QuerySet
from model_utils.managers import InheritanceManager
from taggit.managers import TaggableManager

from armstrong.core.arm_sections.managers import SectionSlugManager
from armstrong.core.arm_sections.models import Section

from . import mixins


class ContentBase(mixins.AuthorsMixin, mixins.PublicationMixin, models.Model):
    """
    The base class providing the basic "armstrong" behavior for a model.

    This is provided as a way to handle cross-model querying.  For example, you
    can use this to query across Article and Video models assuming they both
    extend from a concrete implementation of this class.

    This is *not* a concrete implementation.  This is to avoid having any
    tables created that are not needed.  `armstrong.apps.content`_ provides a
    concrete implementation of this if you want to use it without defining your
    own base Content model.

    .. _armstrong.apps.content: http://github.com/armstrongcms/armstrong.apps.content
    """

    title = models.CharField(max_length=255)
    summary = models.TextField()
    slug = models.SlugField()

    primary_section = models.ForeignKey(Section, null=True)
    sections = models.ManyToManyField(Section, null=True, blank=True,
            related_name="alternates")

    tags = TaggableManager()
    with_section = SectionSlugManager()

    objects = InheritanceManager()

    # TODO: add required primary section
    # TODO: add secondary sections
    class Meta:
        abstract = True
